/*****************************************************************************
 *
 * perfparse.c - Parse Nagios data and store in relational database.
 *
 * Copyright (c) 2004 Ben Clewett (perfparse@clewett.org.uk)
 * Last Modified:   2004-05-28
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <signal.h>
#include <time.h>
#include <glib.h>
#include <sys/stat.h>
#include <libpp_common.h>
#include "common.h"
#include "db_macro_actions.h"
#include "dbms.h"
#include "convert.h"
#include <config.h>
#include <gettext.h>
#include <locale.h>

/* Debug / Output definitions: */
#undef SHOW_SQL
#undef DEBUG

#define IN_SIZE 1024

/* Domain to use: */
#define SERVICE "service"

// #define TESTING


void now_purge();
void my_on_exit();
void delete_deleted_items(void);
void update_policy(void);
void deleteRawData();
void deleteBinData();
void deleteDaysOld();
void print_sql_failure(char *file, int num, const char *sql, const char *error);

/* Accounting */
int  iDeleted = 0;
/* Options */
int  bNoRaw = FALSE;
int  iDeleteDaysOld = 0;

/***************************************
 *
 * Main
 *
 ***************************************/



int main(int argc, char **argv)
{
	int options[] = {
		CONFIG_ITEM_ID_DB_DATABASE,
		CONFIG_ITEM_ID_DB_HOST,
		CONFIG_ITEM_ID_DB_PWD,
		CONFIG_ITEM_ID_DB_USER,
		CONFIG_ITEM_ID_DO_REPORT,
		CONFIG_ITEM_ID_ERROR_LOG,
		CONFIG_ITEM_ID_ERROR_LOG_ROTATE,
		CONFIG_ITEM_ID_NO_TRANSACTIONS,
		CONFIG_ITEM_ID_NB
	};

	setlocale(LC_ALL, "");
	bindtextdomain(PACKAGE, LOCALEDIR);
	textdomain(PACKAGE);

	switch(config_parse_config(argc,argv,_("perfparse-db-purge [options]\n\n"),"\n",options, LOCALSTATEDIR "/perfparse-db-purge.log")) {
		case CONFIG_OPTION_HELP: config_print_help(stdout, _("perfparse-db-purge [options]\n\n"), "\n", options); exit(EXIT_SUCCESS);
		case CONFIG_OPTION_VERSION: config_print_version(stdout, _("perfparse-db-purge [options]\n\n"), "\n"); exit(EXIT_SUCCESS);
		case CONFIG_OPTION_SHOW_CONFIG: config_show_config(stdout, _("perfparse-db-purge [options]\n\n"), "\n", options); exit(EXIT_SUCCESS);
		case CONFIG_OPTION_NONE: break;
	}

	/* Register the error options */
	#ifdef USE_DB_MYSQL
	libpp_mysql_set_sql_failure_callback(print_sql_failure);
	#elif defined USE_DB_POSTGRESQL
	libpp_postgresql_set_sql_failure_callback(print_sql_failure);
	#endif

	/* Connect to Database: */
	now_connect(TRUE);

	/* Register the on_exit: */
	atexit(my_on_exit);

	/* exit if the database format is too old */
	exit_if_db_version_not_correct();

	check_db_integrity();
	
	/* Purge */
	now_purge();

	exit(EXIT_SUCCESS);
}


void my_on_exit()
{

	#ifdef USE_DB_MYSQL
	mysql_close(&db_mysql);
	#elif defined USE_DB_POSTGRESQL
	PQfinish(db_postgresql);
	#endif

}

/***************************************
 *
 * Our DBMS work
 *
 ***************************************/

void now_purge()
{
	if (!config_get_value_as_boolean(CONFIG_ITEM_ID_NO_TRANSACTIONS)) {
		query("START TRANSACTION");
	}

	deleteDaysOld();

	delete_deleted_items();

	update_policy();

	deleteBinData();

	deleteRawData();

	if (!config_get_value_as_boolean(CONFIG_ITEM_ID_NO_TRANSACTIONS)) {
		query("COMMIT");
	}

}

void update_policy(void) {

	#ifdef testing
	#ifdef USE_DB_POSTGRESQL
	int numberOfRows;
	#endif
	#endif

	query_no_return("UPDATE perfdata_host,perfdata_delete_policy "
			"SET perfdata_host.raw_delete_policy = perfdata_delete_policy.delete_policy "
			"WHERE perfdata_host.raw_delete_policy_type = 'template' "
			"AND perfdata_host.raw_delete_policy_name = perfdata_delete_policy.policy_name ");

	query_no_return("UPDATE perfdata_host,perfdata_delete_policy "
			"SET perfdata_host.bin_delete_policy = perfdata_delete_policy.delete_policy "
			"WHERE perfdata_host.bin_delete_policy_type = 'template' "
			"AND perfdata_host.bin_delete_policy_name = perfdata_delete_policy.policy_name ");

	query_no_return("UPDATE perfdata_service,perfdata_delete_policy "
			"SET perfdata_service.raw_delete_policy = perfdata_delete_policy.delete_policy "
			"WHERE perfdata_service.raw_delete_policy_type = 'template' "
			"AND perfdata_service.raw_delete_policy_name = perfdata_delete_policy.policy_name ");

	query_no_return("UPDATE perfdata_service,perfdata_host "
			"SET perfdata_service.raw_delete_policy = perfdata_host.raw_delete_policy "
			"WHERE perfdata_service.raw_delete_policy_type = 'host' "
			"AND perfdata_service.host_name = perfdata_host.host_name ");

	query_no_return("UPDATE perfdata_service_metric,perfdata_delete_policy "
			"SET perfdata_service_metric.bin_delete_policy = perfdata_delete_policy.delete_policy "
			"WHERE perfdata_service_metric.bin_delete_policy_type = 'template' "
			"AND perfdata_service_metric.bin_delete_policy_name = perfdata_delete_policy.policy_name ");

	query_no_return("UPDATE perfdata_service_metric,perfdata_host "
			"SET perfdata_service_metric.bin_delete_policy = perfdata_host.bin_delete_policy "
			"WHERE perfdata_service_metric.bin_delete_policy_type = 'host' "
			"AND perfdata_service_metric.host_name = perfdata_host.host_name ");

#ifdef TESTING

	printf(_("Binary delete policies:\n"));

	g_string_printf(s_SQL, "SELECT host_name, service_description, metric, bin_delete_policy FROM perfdata_service_metric");
	query(s_SQL->str);
	
	#ifdef USE_DB_MYSQL
	while ((result_row = mysql_fetch_row(query_result))) {
	#elif defined USE_DB_POSTGRESQL
	numberOfRows = rows();
	for (currentRow=0;currentRow<numberOfRows;currentRow++){
	#endif
	
		printf("%-20s %-20s %-20s %s\n",
			sData(0), sData(1), sData(2),
			(sData(3)[0]) ? sData(3) : "Never");

	}

	printf(_("\nRaw delete policies:\n"));

	g_string_printf(s_SQL, "SELECT host_name, service_description, raw_delete_policy FROM perfdata_service");
	query(s_SQL->str);
	
	#ifdef USE_DB_MYSQL
	while ((result_row = mysql_fetch_row(query_result))) {
	#elif defined USE_DB_POSTGRESQL
	numberOfRows = rows();
	for (currentRow=0;currentRow<numberOfRows;currentRow++){
	#endif
	
		printf("%-20s %-20s %-20s %s\n",
			sData(0), sData(1), "",
			(sData(2)[0]) ? sData(2) : "Never");

	}

#endif

}


void delete_deleted_items(void)
{

	int iDelete = 0;
	int iCountHost, iCountService, iCountMetric;

#ifdef TESTING
	return;
#endif

	query_no_return("SET AUTOCOMMIT = 1");

	iCountHost = query_int("SELECT COUNT(*) FROM perfdata_host WHERE is_deleted = 1");
	iCountService = query_int("SELECT COUNT(*) FROM perfdata_service WHERE is_deleted = 1");
	iCountMetric = query_int("SELECT COUNT(*) FROM perfdata_service_metric WHERE is_deleted = 1");

	if (iCountHost == 0 && iCountService == 0 && iCountMetric == 0) {
		printf(_("+-----------------------------------------------------------------------------+\n"));
		printf("| %-75s |\n", _("There are no Hosts, Services or Metrics marked for deletion today."));
		fflush(stdout);
		return;
	}

	if (config_get_value_as_boolean(CONFIG_ITEM_ID_DO_REPORT)) {
		printf(_("+-----------------------------------------------------------------------------+\n"));
		printf("| %-75s |\n", _("Deleting data flagged for removal.  This may take some time."));
		printf("| %-75s |\n", _("Please ensure that NO data for these hosts will be added during deletion."));
		printf("| %-75s |\n", _("Record counts will show below."));
		printf(_("+-----------------------------------------------------------------------------+\n"));
		printf("| %-66s %08d |\n", _("Hosts marked for deletion:"), iCountHost);
		printf("| %-66s %08d |\n", _("Services marked for deletion:"), iCountService);
		printf("| %-66s %08d |\n", _("Metrics marked for deletion:"), iCountMetric);
		printf(_("+-----------------------------------------------------------------------------+\n"));
		fflush(stdout);
	}

	/* Delete data */
	/* Because of Referential Integrity, start from outer data tables, work in towards main tables. */

	/* Binary Summary */
	
	if (iCountHost) {

		query_no_return("DELETE perfdata_bin_summary_data "
				"FROM perfdata_bin_summary_data, perfdata_service_metric, perfdata_host "
				"WHERE perfdata_service_metric.host_name = perfdata_host.host_name "
				"AND perfdata_bin_summary_data.metric_id = perfdata_service_metric.metric_id "
				"AND perfdata_host.is_deleted = 1");

		iDelete += iRowsAffected;

		if (config_get_value_as_boolean(CONFIG_ITEM_ID_DO_REPORT)) {
			printf("| %-66s %08d |\n", _("Table: Binary Summary Data, by Host deletion."), iRowsAffected);
			fflush(stdout);
		}

		query_no_return("COMMIT");

	}

	if (iCountService) {

		query_no_return("DELETE perfdata_bin_summary_data "
				"FROM perfdata_bin_summary_data, perfdata_service_metric, perfdata_service "
				"WHERE perfdata_bin_summary_data.metric_id = perfdata_service_metric.metric_id "
				"AND perfdata_service_metric.service_description = perfdata_service.service_description "
				"AND perfdata_service.is_deleted = 1");

		iDelete += iRowsAffected;

		if (config_get_value_as_boolean(CONFIG_ITEM_ID_DO_REPORT)) {
			printf("| %-66s %08d |\n", _("Table: Binary Summary Data, by Service deletion."), iRowsAffected);
			fflush(stdout);
		}

		query_no_return("COMMIT");

	}

	if (iCountMetric) {

		query_no_return("DELETE perfdata_bin_summary_data "
				"FROM perfdata_bin_summary_data, perfdata_service_metric "
				"WHERE perfdata_bin_summary_data.metric_id = perfdata_service_metric.metric_id "
				"AND perfdata_service_metric.is_deleted = 1");
	
		iDelete += iRowsAffected;

		if (config_get_value_as_boolean(CONFIG_ITEM_ID_DO_REPORT)) {
			printf("| %-66s %08d |\n", _("Table: Binary Summary Data, by Metric deletion."), iRowsAffected);
			fflush(stdout);
		}

		query_no_return("COMMIT");

	}

	if (iCountHost) {

		query_no_return("DELETE perfdata_bin_summary_header "
				"FROM perfdata_bin_summary_header, perfdata_service_metric, perfdata_host "
				"WHERE perfdata_service_metric.host_name = perfdata_host.host_name "
				"AND perfdata_bin_summary_header.metric_id = perfdata_service_metric.metric_id "
				"AND perfdata_host.is_deleted = 1");

		iDelete += iRowsAffected;

		if (config_get_value_as_boolean(CONFIG_ITEM_ID_DO_REPORT)) {
			printf("| %-66s %08d |\n", _("Table: Binary Summary Header, by Host deletion"), iRowsAffected);
			fflush(stdout);
		}

		query_no_return("COMMIT");

	}

	if (iCountService) {

		query_no_return("DELETE perfdata_bin_summary_header "
				"FROM perfdata_bin_summary_header, perfdata_service_metric, perfdata_service "
				"WHERE perfdata_bin_summary_header.metric_id = perfdata_service_metric.metric_id "
				"AND perfdata_service_metric.service_description = perfdata_service.service_description "
				"AND perfdata_service.is_deleted = 1");

		iDelete += iRowsAffected;

		if (config_get_value_as_boolean(CONFIG_ITEM_ID_DO_REPORT)) {
			printf("| %-66s %08d |\n", _("Table: Binary Summary Header, by Service deletion"), iRowsAffected);
			fflush(stdout);
		}

		query_no_return("COMMIT");

	}

	if (iCountMetric) {

		query_no_return("DELETE perfdata_bin_summary_header "
				"FROM perfdata_bin_summary_header, perfdata_service_metric "
				"WHERE perfdata_bin_summary_header.metric_id = perfdata_service_metric.metric_id "
				"AND perfdata_service_metric.is_deleted = 1");
	
		iDelete += iRowsAffected;

		if (config_get_value_as_boolean(CONFIG_ITEM_ID_DO_REPORT)) {
			printf("| %-66s %08d |\n", _("Table: Binary Summary Header, by Metric deletion"), iRowsAffected);
			fflush(stdout);
		}

		query_no_return("COMMIT");

	}

	
	/* Raw Summary */

	if (iCountHost) {

		query_no_return("DELETE perfdata_raw_summary_data "
				"FROM perfdata_raw_summary_data, perfdata_service, perfdata_host "
				"WHERE perfdata_service.host_name = perfdata_host.host_name "
				"AND perfdata_raw_summary_data.service_id = perfdata_service.service_id "
				"AND perfdata_host.is_deleted = 1");

		iDelete += iRowsAffected;

		if (config_get_value_as_boolean(CONFIG_ITEM_ID_DO_REPORT)) {
			printf("| %-66s %08d |\n", _("Table: Raw Summary Data, by Host deletion."), iRowsAffected);
			fflush(stdout);
		}

		query_no_return("COMMIT");

	}

	if (iCountService) {

		query_no_return("DELETE perfdata_raw_summary_data "
				"FROM perfdata_raw_summary_data, perfdata_service "
				"WHERE perfdata_raw_summary_data.service_id = perfdata_service.service_id "
				"AND perfdata_service.is_deleted = 1");

		iDelete += iRowsAffected;

		if (config_get_value_as_boolean(CONFIG_ITEM_ID_DO_REPORT)) {
			printf("| %-66s %08d |\n", _("Table: Raw Summary Data, by Service deletion."), iRowsAffected);
			fflush(stdout);
		}

		query_no_return("COMMIT");

	}


	if (iCountHost) {

		query_no_return("DELETE perfdata_raw_summary "
				"FROM perfdata_raw_summary, perfdata_service, perfdata_host "
				"WHERE perfdata_service.host_name = perfdata_host.host_name "
				"AND perfdata_raw_summary.service_id = perfdata_service.service_id "
				"AND perfdata_host.is_deleted = 1");

		iDelete += iRowsAffected;

		if (config_get_value_as_boolean(CONFIG_ITEM_ID_DO_REPORT)) {
			printf("| %-66s %08d |\n", _("Table: Raw Summary Header, by Host deletion."), iRowsAffected);
			fflush(stdout);
		}

		query_no_return("COMMIT");

	}


	if (iCountService) {

		query_no_return("DELETE perfdata_raw_summary "
				"FROM perfdata_raw_summary, perfdata_service "
				"WHERE perfdata_raw_summary.service_id = perfdata_service.service_id "
				"AND perfdata_service.is_deleted = 1");
	
		iDelete += iRowsAffected;

		if (config_get_value_as_boolean(CONFIG_ITEM_ID_DO_REPORT)) {
			printf("| %-66s %08d |\n", _("Table: Raw Summary Header, by Service deletion."), iRowsAffected);
			fflush(stdout);
		}

		query_no_return("COMMIT");

	}



	/* Raw Data */

	if (iCountHost) {

		query_no_return("DELETE perfdata_service_raw "
				"FROM perfdata_service_raw,perfdata_host "
				"WHERE perfdata_service_raw.host_name = perfdata_host.host_name "
				"AND perfdata_host.is_deleted = 1");

		iDelete += iRowsAffected;

		if (config_get_value_as_boolean(CONFIG_ITEM_ID_DO_REPORT)) {
			printf("| %-66s %08d |\n", _("Table: Raw Data, by Host deletion."), iRowsAffected);
			fflush(stdout);
		}

		query_no_return("COMMIT");

	}

	if (iCountService) {

		query_no_return("DELETE perfdata_service_raw "
				"FROM perfdata_service_raw,perfdata_service "
				"WHERE perfdata_service_raw.service_description = perfdata_service.service_description "
				"AND perfdata_service_raw.host_name = perfdata_service.host_name "
				"AND perfdata_service.is_deleted = 1");

		iDelete += iRowsAffected;

		if (config_get_value_as_boolean(CONFIG_ITEM_ID_DO_REPORT)) {
			printf("| %-66s %08d |\n", _("Table: Raw Data, by Service deletion."), iRowsAffected);
			fflush(stdout);
		}

		query_no_return("COMMIT");

	}


	/* Binary Data */

	if (iCountHost) {

		query_no_return("DELETE perfdata_service_bin "
				"FROM perfdata_service_bin,perfdata_host "
				"WHERE perfdata_service_bin.host_name = perfdata_host.host_name "
				"AND perfdata_host.is_deleted = 1");

		iDelete += iRowsAffected;

		if (config_get_value_as_boolean(CONFIG_ITEM_ID_DO_REPORT)) {
			printf("| %-66s %08d |\n", _("Table: Binary Data, by Host deletion."), iRowsAffected);
			fflush(stdout);
		}

		query_no_return("COMMIT");

	}


	if (iCountService) {

		query_no_return("DELETE perfdata_service_bin "
				"FROM perfdata_service_bin,perfdata_service "
				"WHERE perfdata_service_bin.service_description = perfdata_service.service_description "
				"AND perfdata_service_bin.host_name = perfdata_service.host_name "
				"AND perfdata_service.is_deleted = 1");

		iDelete += iRowsAffected;

		if (config_get_value_as_boolean(CONFIG_ITEM_ID_DO_REPORT)) {
			printf("| %-66s %08d |\n", _("Table: Binary Data, by Service deletion"), iRowsAffected);
			fflush(stdout);
		}

		query_no_return("COMMIT");

	}
	

	if (iCountMetric) {

		query_no_return("DELETE perfdata_service_bin "
				"FROM perfdata_service_bin,perfdata_service_metric "
				"WHERE perfdata_service_bin.service_description = perfdata_service_metric.service_description "
				"AND perfdata_service_bin.metric = perfdata_service_metric.metric "
				"AND perfdata_service_bin.host_name = perfdata_service_metric.host_name "
				"AND perfdata_service_metric.is_deleted = 1");

		iDelete += iRowsAffected;

		if (config_get_value_as_boolean(CONFIG_ITEM_ID_DO_REPORT)) {
			printf("| %-66s %08d |\n", _("Table: Binary Data, by Metric deletion."), iRowsAffected);
			fflush(stdout);
		}

		query_no_return("COMMIT");

	}

	/* Metrics */


	if (iCountHost) {

		query_no_return("DELETE perfdata_service_metric "
				"FROM perfdata_service_metric,perfdata_host "
				"WHERE perfdata_service_metric.host_name = perfdata_host.host_name "
				"AND perfdata_host.is_deleted = 1");

		iDelete += iRowsAffected;

		if (config_get_value_as_boolean(CONFIG_ITEM_ID_DO_REPORT)) {
			printf("| %-66s %08d |\n", _("Table: Metrics, by Host deletion."), iRowsAffected);
			fflush(stdout);
		}

		query_no_return("COMMIT");

	}

	if (iCountService) {

		query_no_return("DELETE perfdata_service_metric "
				"FROM perfdata_service,perfdata_service_metric "
				"WHERE perfdata_service.service_description = perfdata_service_metric.service_description "
				"AND perfdata_service.host_name = perfdata_service_metric.host_name "
				"AND perfdata_service.is_deleted = 1");

		iDelete += iRowsAffected;

		if (config_get_value_as_boolean(CONFIG_ITEM_ID_DO_REPORT)) {
			printf("| %-66s %08d |\n", _("Table: Metrics, by Service deletion."), iRowsAffected);
			fflush(stdout);
		}

		query_no_return("COMMIT");

	}

	if (iCountMetric) {

		query_no_return("DELETE FROM perfdata_service_metric "
				"WHERE is_deleted = 1");

		iDelete += iRowsAffected;

		if (config_get_value_as_boolean(CONFIG_ITEM_ID_DO_REPORT)) {
			printf("| %-66s %08d |\n", _("Table: Metrics, by Metric deletion."), iRowsAffected);
			fflush(stdout);
		}

		query_no_return("COMMIT");

	}

	/* Service */
	
	if (iCountHost) {

		query_no_return("DELETE perfdata_service "
				"FROM perfdata_service,perfdata_host "
				"WHERE perfdata_service.host_name = perfdata_host.host_name "
				"AND perfdata_host.is_deleted = 1");

		iDelete += iRowsAffected;

		if (config_get_value_as_boolean(CONFIG_ITEM_ID_DO_REPORT)) {
			printf("| %-66s %08d |\n", _("Table: Services, by Host deletion."), iRowsAffected);
			fflush(stdout);
		}

		query_no_return("COMMIT");

	}


	if (iCountService) {

		query_no_return("DELETE FROM perfdata_service WHERE is_deleted = 1");
	
		iDelete += iRowsAffected;

		if (config_get_value_as_boolean(CONFIG_ITEM_ID_DO_REPORT)) {
			printf("| %-66s %08d |\n", _("Table: Services, by Service deletion."), iRowsAffected);
			fflush(stdout);
		}

		query_no_return("COMMIT");

	}


	/* Hosts */

	if (iCountHost) {

		query_no_return("DELETE FROM perfdata_host WHERE is_deleted = 1");

		iDelete += iRowsAffected;

		if (config_get_value_as_boolean(CONFIG_ITEM_ID_DO_REPORT)) {
			printf("| %-66s %08d |\n", _("Table: Host, by Host deletion."), iRowsAffected);
			fflush(stdout);
		}

		query_no_return("COMMIT");

	}


	if (config_get_value_as_boolean(CONFIG_ITEM_ID_DO_REPORT)) {
		printf(_("+-----------------------------------------------------------------------------+\n"));
		printf(_("| %-66s %08d |\n"), _("Total Records deleted:"), iDelete);
	}

}

void deleteDaysOld()
{
	int iDelete = 0;

	if (!iDeleteDaysOld) return;

	/* Delete all data more than iDeleteDaysOld days old. */

	if (config_get_value_as_boolean(CONFIG_ITEM_ID_DO_REPORT)) {
		printf(_("+-----------------------------------------------------------------------------+\n"));
		printf(_("| Deleting data more than: %4d days old:  "), iDeleteDaysOld);
		fflush(stdout);
	}

	g_string_printf(s_SQL, "DELETE FROM perfdata_service_raw WHERE ctime < NOW() - INTERVAL %d DAY", 
		iDeleteDaysOld);
	query_no_return(s_SQL->str);
	iDelete += iRowsAffected;
	iDeleted += iDelete;

	g_string_printf(s_SQL, "DELETE FROM perfdata_service_bin WHERE ctime < NOW() - INTERVAL %d DAY", 
		iDeleteDaysOld);
	query_no_return(s_SQL->str);
	iDelete += iRowsAffected;
	iDeleted += iDelete;

	if (config_get_value_as_boolean(CONFIG_ITEM_ID_DO_REPORT)) {
		printf(_(" %7d Records deleted.          |\n"), iDelete);
//		printf(_("+-----------------------------------------------------------------------------+\n"));
	}

}

void deleteBinData()
{

	int bHeader = FALSE;
	#ifdef USE_DB_POSTGRESQL
	int numberOfRows;
	#endif
	

	query("SELECT host_name, service_description, metric, bin_delete_policy FROM perfdata_service_metric WHERE bin_delete_policy IS NOT NULL");
	#ifdef USE_DB_MYSQL
	while ((result_row = mysql_fetch_row(query_result))) {
	#elif defined USE_DB_POSTGRESQL
	numberOfRows = rows();
	for (currentRow=0;currentRow<numberOfRows;currentRow++){
	#endif	

		if (!bHeader) {
			bHeader = TRUE;
			if (config_get_value_as_boolean(CONFIG_ITEM_ID_DO_REPORT)) {
				printf(_("+-----------------------------------------------------------------------------+\n"));
				printf(_("|  Binary Delete Policy                                                       |\n"));
				printf(_("+---------------------------------------------------+--------------+----------+\n"));
				printf(_("| Metric                                            | Policy       | Deleted  |\n"));
				printf(_("+---------------------------------------------------+--------------+----------+\n"));
			}
		}

		if (config_get_value_as_boolean(CONFIG_ITEM_ID_DO_REPORT)) {
			g_string_printf(s_SQL, "%s, %s, %s",
				sData(0), sData(1), sData(2));
			if (iData(3) > 0) {
				printf(_("| %-49s | To %4d days | "),
					s_SQL->str, iData(3));
			} else {
				printf(_("| %-49s | All data     | "),
					s_SQL->str);
			}
			fflush(stdout);
		}

		g_string_printf(s_SQL, "DELETE FROM perfdata_service_bin WHERE host_name = '%s'",
			sql_escape(sData(0),PERCENT_ESCAPE_CHAR));
		g_string_append_printf(s_SQL, " AND service_description = '%s'",
			sql_escape(sData(1),PERCENT_ESCAPE_CHAR));
		g_string_append_printf(s_SQL, " AND metric = '%s'",
			sql_escape(sData(2),PERCENT_ESCAPE_CHAR));

		if (iData(3) > 0) {
			/* Delete for a time period */
			g_string_append_printf(s_SQL, " AND ctime < NOW() - INTERVAL %d DAY", 
				iData(3));
		}

#ifndef TESTING
		query_no_return(s_SQL->str);
		iDeleted += iRowsAffected;
#endif

		if (config_get_value_as_boolean(CONFIG_ITEM_ID_DO_REPORT)) {
			printf(_("%8d |\n"), iRowsAffected);
		}

	}
	if (config_get_value_as_boolean(CONFIG_ITEM_ID_DO_REPORT)) {
		printf(_("+---------------------------------------------------+--------------+----------+\n"));
	}

}


void deleteRawData()
{

	int bHeader = FALSE;
	#ifdef USE_DB_POSTGRESQL
	int numberOfRows;
	#endif


	query("SELECT host_name, service_description, raw_delete_policy FROM perfdata_service WHERE raw_delete_policy IS NOT NULL");
	#ifdef USE_DB_MYSQL
	while ((result_row = mysql_fetch_row(query_result))) {
	#elif defined USE_DB_POSTGRESQL
	numberOfRows = rows();
	for (currentRow=0;currentRow<numberOfRows;currentRow++){
	#endif	
	
		if (!bHeader) {
			bHeader = TRUE;
			if (config_get_value_as_boolean(CONFIG_ITEM_ID_DO_REPORT)) {
//				printf(_("+-----------------------------------------------------------------------------+\n"));
				printf(_("|  Raw Data Delete Policy                                                     |\n"));
				printf(_("+---------------------------------------------------+--------------+----------+\n"));
				printf(_("| Service                                           | Policy       | Deleted  |\n"));
				printf(_("+---------------------------------------------------+--------------+----------+\n"));
			}
		}

		if (config_get_value_as_boolean(CONFIG_ITEM_ID_DO_REPORT)) {
			g_string_printf(s_SQL, "%s, %s",
				sData(0), sData(1));
			if (iData(2) > 0) {
				printf(_("| %-49s | To %4d days | "),
					s_SQL->str, iData(2));
			} else {
				printf(_("| %-49s | All data     | "),
					s_SQL->str);
			}
			fflush(stdout);
		}

		g_string_printf(s_SQL, "DELETE FROM perfdata_service_raw WHERE host_name = '%s'",
			sql_escape(sData(0),PERCENT_ESCAPE_CHAR));
		g_string_append_printf(s_SQL, " AND service_description = '%s'",
			sql_escape(sData(1),PERCENT_ESCAPE_CHAR));

		if (iData(2) > 0) {
			/* Delete for a time period */
			g_string_append_printf(s_SQL, " AND ctime < NOW() - INTERVAL %d DAY", 
				iData(2));
		}

#ifndef TESTING
		query_no_return(s_SQL->str);
		iDeleted += iRowsAffected;
#endif

		if (config_get_value_as_boolean(CONFIG_ITEM_ID_DO_REPORT)) {
			printf("%8d |\n", iRowsAffected);
		}

	}

	if (config_get_value_as_boolean(CONFIG_ITEM_ID_DO_REPORT)) {
		printf(_("+---------------------------------------------------+--------------+----------+\n"));
	}

}


void print_sql_failure(char*file,int num, const char *sql, const char *error)
{

        printf(_("An error occured with the SQL:\n\n  \"%s\"\n\nFailure Message:\n\n  \"%s\"\n\n"),
                sql, error);

        exit(EXIT_FAILURE);

}

