/*****************************************************************************
 *
 * log_parser.h - Parse Nagios data and store in relational database.
 *
 * Copyright (c) 2004 Yves Mettier (ymettier@libertysurf.fr)
 * Last Modified:   2004-08-25
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#ifndef __LOG_PARSER__
#define __LOG_PARSER__

#include <time.h>
#include <math.h>

#include <config.h>

#ifdef OS_SOLARIS
#include <ieeefp.h>
#endif /* OS_SOLARIS */

#ifndef INFINITY
#define INFINITY HUGE_VAL
#endif

#ifndef NAN
#if defined (OS_SOLARIS)
#define NAN FP_QNAN
#endif /* OS_SOLARIS */
#if defined (OS_LINUX) || defined (OS_OPENBSD) || defined (OS_DARWIN)
/* From /usr/include/bits/nan.h from a RedHat Linux System */
# if __BYTE_ORDER == __BIG_ENDIAN
#  define __nan_bytes           { 0x7f, 0xc0, 0, 0 }
# endif
# if __BYTE_ORDER == __LITTLE_ENDIAN
#  define __nan_bytes           { 0, 0, 0xc0, 0x7f }
# endif
static union { unsigned char __c[4]; float __d; } __nan_union = { __nan_bytes };
# define NAN    (__nan_union.__d)
#endif /* OS_LINUX */
#endif /* NAN */

#define UNIT_LENGTH 20

typedef enum {
	LOG_LINE_TIMET = 0,
	LOG_LINE_HOSTNAME = 1,
	LOG_LINE_SERVICEDESC = 2,
	LOG_LINE_OUTPUT = 3,
	LOG_LINE_SERVICESTATE = 4,
	LOG_LINE_PERFDATA = 5,
	LOG_LINE_MACRO_NB = 6
} log_line_e;

typedef enum {
	SERVICE_STATE_OK = 0,
	SERVICE_STATE_WARNING = 1,
	SERVICE_STATE_CRITICAL = 2,
	SERVICE_STATE_UNKNOWN = 3,
	SERVICE_STATE_UNSET = -1
} service_state_e;

typedef struct {
	char *macro[LOG_LINE_MACRO_NB];
	int macro_len[LOG_LINE_MACRO_NB];
	time_t time;
	service_state_e service_state;
} log_line_t;

typedef enum {
	PERF_FIELDS_TYPE_STRING,
	PERF_FIELDS_TYPE_PERF
} perf_fields_type_e;

typedef struct {
	char *label;
	int label_len;
	char *value;
	int value_len;
	int perf_offset;
	perf_fields_type_e type;
} perf_fields_t;

typedef enum {
	PERF_VALUE_RANGE_WARNING_RANGE_INSIDE = 	1<<1,
	PERF_VALUE_RANGE_CRITICAL_RANGE_INSIDE = 	1<<2,
} perf_value_flags_e;

typedef enum {
	PERF_VALUE_VALUE = 0,
	PERF_VALUE_WARN_START = 1,
	PERF_VALUE_WARN_END = 2,
	PERF_VALUE_CRIT_START = 3,
	PERF_VALUE_CRIT_END = 4,
	PERF_VALUE_MIN = 5,
	PERF_VALUE_MAX = 6
} perf_value_e;

typedef struct {
	double d[7]; /* value, warn_start, warn_end, crit_start, crit_end, min, max */
	int ranges_type;
	char unit[UNIT_LENGTH];
	service_state_e metric_state;
	const char *range;
} perf_value_t;

log_line_t *log_line_new(void);
void log_line_free(log_line_t*p);
perf_fields_t *perf_fields_new(void);
int perf_get_first (log_line_t *data, perf_fields_t *p);
int perf_get_next(log_line_t *data, perf_fields_t *p);
perf_value_t *perf_value_new(void);
int perfstring_to_perfvalue(const char*str, perf_value_t *p);
int line_split(char *line, log_line_t *data);
#endif
