/*****************************************************************************
 *
 * clean_tools.c - Parse Nagios data and store in relational database.
 *
 * Copyright (c) 2004 Yves Mettier (ymettier@libertysurf.fr)
 * Last Modified:   2004-12-14
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <dirent.h>
#include <errno.h>
#include <time.h>
#include <glib.h>
#include <libpp_common.h>
#include <config.h>
#include <gettext.h>
#include <zlib.h>

int compress_file(const char*name) {
	char buffer[BUFSIZ];
	gzFile*gz;
	int fd;
	unsigned char sig[2];
	int l;

	if(-1 == (fd = open(name, O_RDONLY))) {
		pp_log(__FILE__,__LINE__, _("Could not open file '%s'"), name);
		l = -1;
	} else {
		/* Test is the file is a compressed file with gzip */
		read(fd,sig,2);
		if((sig[0] != 0037) || (sig[1] != 0213)) {
			char *name_gz = g_strdup_printf("%s.gz",name);
			if(NULL != (gz = gzopen(name_gz, "wb9"))) {
				lseek(fd,0,SEEK_SET);
				while(0 < (l = read(fd,buffer,BUFSIZ))) {
					if(0 == gzwrite(gz,buffer,l)) {l = -2; break;}
				}
				gzclose(gz);
				if(l == 0) {
					unlink(name);
				} else if(l == -1) {
					pp_log(__FILE__,__LINE__, _("An error occured while reading file '%s' (%s)"), name, strerror(errno));
					unlink(name_gz);
				} else if(l == -2) {
					pp_log(__FILE__,__LINE__, _("An error occured while writing file '%s'"), name_gz);
				} else {
					pp_log(__FILE__,__LINE__, _("An error occured while compressing file '%s'. This should not occur. This is a bug. Please report it to the development team. (l=%d)"), name_gz,l);
					l = -3;
				}
			} else {
				l = -4;
			}
			g_free(name_gz);
		} else {
			l = 1;
		}
		close(fd);
	}
	return(l);
}

int check_dir_for_files_to_remove_or_compress_between_2_tm(time_t tm_limit_for_remove, time_t tm_limit_for_compress, const char*path, const char*file_name_pattern) {
	struct stat buf;
	struct dirent *dp;
	DIR*dh;
	GString *name;
	int path_len;
	int nb_files, nb_dirs;
	int file_name_pattern_len = file_name_pattern?strlen(file_name_pattern):0;

	name = g_string_new(path);
	name = g_string_append_c(name, '/');
	path_len = name->len;


	if(NULL == (dh = opendir(path))) {
		pp_log(__FILE__,__LINE__, _("Could not open directory '%s'"), path);
		return(-1);
	}
	nb_files = 0;
	/* Check files first */
	while(NULL != (dp = readdir(dh))) { 
		if(strcmp(dp->d_name,".") && strcmp(dp->d_name,"..")) {
			name = g_string_truncate(name, path_len);
			name = g_string_append(name, dp->d_name);
			if((file_name_pattern == NULL) || (0 == strncmp(file_name_pattern,dp->d_name,file_name_pattern_len))) {
				if(0 == stat(name->str,&buf)) {
					if(buf.st_mode & S_IFREG) {
						if(buf.st_mtime < tm_limit_for_remove) {
							/* Test if the file is too old */
							unlink(name->str);
							rewinddir(dh);
							nb_files = 0;
						} else if(buf.st_mtime < tm_limit_for_compress) {
							/* Compression of the file */
							int l = compress_file(name->str);
							if(l == 0) {
								rewinddir(dh);
								nb_files = 0;
							} else if(l == 1) {
								nb_files++;
							} else {
								closedir(dh);
								exit(EXIT_FAILURE);
							}
						} else {
							/* Don't do anything but count this file */
							nb_files++;
						}
					} else {
						/* Don't do anything but count this file */
						nb_files++;
					}
				}
			} else {
				/* Don't do anything but count this file */
				nb_files++;
			}
		}
	}
	rewinddir(dh);
	nb_dirs = 0;
	/* Check directories */
	while(NULL != (dp = readdir(dh))) { 
		if(strcmp(dp->d_name,".") && strcmp(dp->d_name,"..")) {
			name = g_string_truncate(name, path_len);
			name = g_string_append(name, dp->d_name);
			if(0 == stat(name->str,&buf)) {
				if(buf.st_mode & S_IFDIR) {
					int n;
					n = check_dir_for_files_to_remove_or_compress_between_2_tm(tm_limit_for_remove,tm_limit_for_compress,name->str,file_name_pattern);
					if(n == 0) {
						if(rmdir(name->str)) {
							nb_dirs++;
						} else {
							rewinddir(dh);
							nb_dirs = 0;
						}
					} else {
						nb_dirs++;
					}
				}
			}
		}
	}
	closedir(dh);
	g_string_free(name,TRUE);
	return(nb_files+nb_dirs);
}

int check_dir_for_files_to_remove_or_compress(int nb_days, const char*path, const char*pattern) {
	time_t tm_limit_for_compress, tm_limit_for_remove;
	time_t tm;
	struct tm *stm;

	tm = time(NULL);
	stm = localtime(&tm);
	stm->tm_sec = 0;
	stm->tm_min = 0;
	stm->tm_hour = 0;
	tm_limit_for_compress = mktime(stm);
	if(nb_days >= 0) tm_limit_for_remove = tm_limit_for_compress - 86400 * nb_days;
	else tm_limit_for_remove = 0; /* remove nothing */

	return( check_dir_for_files_to_remove_or_compress_between_2_tm(tm_limit_for_remove,tm_limit_for_compress, path,pattern));
}
