/*****************************************************************************
 *
 * config_file.c - API to the config file used by PerfParse suit.
 *
 * Copyright (c) 2004 Yves Mettier <ymettier@libertysurf.fr>
 * Last Modified:   2004-06-11
 * Copyright change with Ben Clewett agreement (2004-07-28)
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include <gettext.h>
#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#if defined OS_SOLARIS
#include <netdb.h>
#endif
#include <sys/types.h>
#include <regex.h>
#include <assert.h>
#include <glib.h>
#include "libpp_common.h"

/*
 * Configuration variables.
 *
 */
config_item_t config_definition[CONFIG_ITEM_ID_NB];
config_item_t *config;


char *my_gethostname(void)
{
	char *s=NULL;
#if defined (OS_SOLARIS)
	s = malloc(sizeof(char)*MAXHOSTNAMELEN);
	gethostname(s,MAXHOSTNAMELEN);
	s[MAXHOSTNAMELEN-1] = '\0';
#elif defined (OS_LINUX) || defined (OS_OPENBSD) || defined (OS_DARWIN)
	int l=1024;
	while(!s) {
		s = malloc(sizeof(char)*l);
		if(gethostname(s,l)) {
			if(errno == EINVAL) {
				free(s);
				s = NULL;
				l += 1024;
			}
		}
	}
#endif
	return s;
}

int strtoboolean(const char*value) {
	if(!value)return(0);
	switch(value[0]) {
		case '1' : return(1);
		case 'Y' : return(1);
		case 'y' : return(1);
		case 'O' : return(1); /* French for 'yes' is 'oui' */
		case 'o' : return(1);
		default: return(0);
	}
}

#define config_item_add(cd, x, skey, svalue, sdesc, ldesc, sopt, lopt, ttype, bin_config_file) \
	cd[x].key = strdup(skey); \
	cd[x].value = svalue?strdup(svalue):NULL; \
	cd[x].short_description = strdup(sdesc); \
	cd[x].long_description = strdup(ldesc); \
	cd[x].short_option = sopt?strdup(sopt):NULL; \
	cd[x].long_option = lopt?strdup(lopt):NULL; \
	cd[x].integer = svalue?strtol(svalue,NULL,10):0; \
	cd[x].boolean = strtoboolean(svalue); \
	cd[x].in_config_file = bin_config_file; \
	cd[x].type = ttype;


void config_definition_init(const char*default_error_log) {
	int i;
	config = config_definition;
	for(i=0; i<CONFIG_ITEM_ID_NB; i++) config_definition[i].key = NULL;

	config_item_add(config_definition, CONFIG_ITEM_ID_ERROR_LOG, "Error_Log", default_error_log, _("Perfparse log"), _("File where Perfparse logs messages"), "-e", "--error-log", TYPE_STRING, 1);
	config_item_add(config_definition, CONFIG_ITEM_ID_ERROR_LOG_ROTATE, "Error_Log_Rotate", "yes", _("Perfparse log rotate"), _("Rotate Perfparse log files"), NULL, NULL,TYPE_BOOLEAN, 1);
	config_item_add(config_definition, 	CONFIG_ITEM_ID_ERROR_LOG_KEEP_N_DAYS, "Error_Log_Keep_N_Days", "7", _("Keep N days of error log"), _("Keep N days of error log. Compress recent logs and remove too old ones"), NULL, NULL,TYPE_INT, 1);
	config_item_add(config_definition, CONFIG_ITEM_ID_DROP_FILE,  "Drop_File", "/tmp/perfparse.drop", _("File for dropped lines"), _("When perfparse cannot parse a line, it drops it to that file"), NULL, "--drop-file", TYPE_STRING, 1);
	config_item_add(config_definition, 	CONFIG_ITEM_ID_DROP_FILE_ROTATE, "Drop_File_Rotate", "no", "", "", NULL, NULL,TYPE_BOOLEAN, 1);
	config_item_add(config_definition, 	CONFIG_ITEM_ID_DROP_FILE_KEEP_N_DAYS, "Drop_File_Keep_N_Days", "7", _("Keep N days of drop file log"), _("Keep N days of drop file log. Compress recent logs and remove too old ones"), NULL, NULL,TYPE_INT, 1);
	config_item_add(config_definition, 	CONFIG_ITEM_ID_SERVER_PORT, "Server_Port", "1976", _("Server port"), _("Port for perfparsed server\n# Put 0 or \"\" to disable the server"), "-p", "--port", TYPE_INT, 1);
	config_item_add(config_definition, 	CONFIG_ITEM_ID_SERVICE_LOG, "Service_Log", "/usr/local/nagios/var/serviceperf.log", _("Input log source (from nagios or perfparsed)"), _("Log source from nagios (or other tools) that perfparse will scan\n# Authorized values: a file name, '-' for stdin, '|' for a fifo and '>' for a host:port socket\n# For sockets, a command 'history' will be sent before retreiving the data"), "-l", "--serviceperf-log", TYPE_STRING, 1);
	config_item_add(config_definition, 	CONFIG_ITEM_ID_SERVICE_LOG_SAVE_POSITION, "Service_Log_Save_Position", "yes", _("Save the read position"), _("Save the read position in the nagios log file ? If yes, perfparse will start from that position instead of from the beginning"), NULL, NULL, TYPE_BOOLEAN, 1);
	config_item_add(config_definition, 	CONFIG_ITEM_ID_SERVICE_LOG_POSITION_MARK_PATH, "Service_Log_Position_Mark_Path", "" , _("Position mark file path"), _("Path for files containing the read position for nagios log files"), NULL, NULL, TYPE_STRING, 1);
	config_item_add(config_definition, 	CONFIG_ITEM_ID_HISTORY_START_TM, "History_Start_Tm", "-86400", _("Start timestamp for history retreiving"), _("Start timestamp for history retreiving (positive is absolute, negative is relative to end tm)"), NULL, "--history_start_tm", TYPE_INT, 1);
	config_item_add(config_definition, 	CONFIG_ITEM_ID_HISTORY_END_TM, "History_End_Tm", "-30", _("End timestamp for history retreiving"), _("End timestamp for history retreiving (positive is absolute, negative is relative to Now)"), NULL, "--history_end_tm", TYPE_INT, 1);
	config_item_add(config_definition, 	CONFIG_ITEM_ID_SHOW_STATUS_BAR, "Show_Status_Bar", "no", _("Show status bar"), _("Show status bar when running"), "-s", "--status-bar",TYPE_BOOLEAN, 1);
	config_item_add(config_definition, 	CONFIG_ITEM_ID_DO_REPORT, "Do_Report", "no", _("Print a report"), _("Print a report at the end of the processing"), "-r", "--report",TYPE_BOOLEAN, 1);
	config_item_add(config_definition, 	CONFIG_ITEM_ID_DEFAULT_USER_PERMISSIONS_POLICY, "Default_user_permissions_Policy", "rw", _("Policy for default user"), _("Policy for default user"), NULL, NULL, TYPE_STRING, 1);
	config_item_add(config_definition, 	CONFIG_ITEM_ID_DEFAULT_USER_PERMISSIONS_SUMMARY, "Default_user_permissions_Summary", "rw", _("Summary Permissions for default user"), _("Summary Permissions for default user"), NULL, NULL, TYPE_STRING, 1);
	config_item_add(config_definition, 	CONFIG_ITEM_ID_DEFAULT_USER_PERMISSIONS_HOST_GROUP, "Default_user_permissions_Host_groups", "rw", _("Default perms for host groups"), _("Default permissions for host groups"), NULL, NULL, TYPE_STRING, 1);
	config_item_add(config_definition, 	CONFIG_ITEM_ID_DAEMONIZE, "Daemonize", "no", _("Daemonize perfparsed"), _("Run perfparsed as a daemon"), "-d", "--daemonize", TYPE_BOOLEAN, 1);
	config_item_add(config_definition, 	CONFIG_ITEM_ID_DAEMON_LOCK, "Daemon_Lock", LOCALSTATEDIR "/perfparsed.lock", _("Lock file for perfparsed"), _("Lock file for perfparsed"), NULL, NULL, TYPE_STRING, 1);
	config_item_add(config_definition, 	CONFIG_ITEM_ID_PERIODIC_CLEANUP, "Periodic_Cleanup", "yes", _("Perform periodic cleanup "), _("Perform some periodic cleanup every day"), NULL, "--periodic_cleanup", TYPE_BOOLEAN, 1);
	config_item_add(config_definition, 	CONFIG_ITEM_ID_PERIODIC_CLEANUP_LOCK, "Periodic_Cleanup_Lock", LOCALSTATEDIR "/perfparsed_periodic_cleanup.lock", _("Lock file for perfparsed periodic cleanup process"), _("Lock file for perfparsed periodic cleanup process"), NULL, NULL, TYPE_STRING, 1);
	config_item_add(config_definition, 	CONFIG_ITEM_ID_PERIODIC_CLEANUP_HOUR, "Periodic_Cleanup_Hour", "0230", _("Periodic cleanup hour (HHMM)"), _("Perform some periodic cleanup every day at HHMM"), NULL, "--periodic_cleanup_hour", TYPE_INT, 1);
	config_item_add(config_definition, 	CONFIG_ITEM_ID_STORAGE_MODULES_DIR, "Storage_Modules_Dir", LIBDIR, _("Path where storage modules are"), _("Path where storage modules are"), NULL, "--storage_modules_dir", TYPE_STRING, 1);
	config_item_add(config_definition, 	CONFIG_ITEM_ID_STORAGE_MODULES_LOAD, "Storage_Modules_Load", "", _("Modules to load (CSV)"), _("Modules to load (Coma separated values)"), NULL, "--storage_modules_load", TYPE_STRING, 1);
	config_item_add(config_definition, 	CONFIG_ITEM_ID_STORAGE_MODULES_STATUS_FILE, "Storage_Modules_Status_File", LOCALSTATEDIR "/storage_modules.status", _("File to contain storage modules status"), _("File to contain Storage Modules Status"), NULL, NULL, TYPE_STRING, 1);
	config_item_add(config_definition, 	CONFIG_ITEM_ID_STORAGE_FILE_OUTPUT_LOG_FILENAME, "Output_Log_Filename", LOCALSTATEDIR "/perfparse_output_log", _("Output log filename or path"), _("Output log filename for the file (or directory) that will contain the line that perfparse parses"), NULL, "--output_file_filename", TYPE_STRING, 1);
	config_item_add(config_definition, 	CONFIG_ITEM_ID_STORAGE_FILE_OUTPUT_LOG_ROTATE, "Output_Log_Rotate", "yes", _("Rotate output log"), _("Rotate output log ? If yes, a directory Output_Log_Rotate/YYYY/MM/YYYYMMDD.log will be created"), NULL, "--output_file_rotate",TYPE_BOOLEAN, 1);
	config_item_add(config_definition, 	CONFIG_ITEM_ID_STORAGE_FILE_OUTPUT_LOG_KEEP_N_DAYS, "Output_Log_Keep_N_Days", "7", _("Keep N days of output log"), _("Keep N days of output log. Compress recent logs and remove too old ones"), NULL, NULL,TYPE_INT, 1);
	config_item_add(config_definition, 	CONFIG_ITEM_ID_STORAGE_GNUPLOT_OUTPUT_FILE, "Gnuplot_Output_File", "-", _("Output for gnuplot"), _("Output file for gnuplot output. '-' for stdout"), NULL, NULL,TYPE_STRING, 1);
	config_item_add(config_definition, CONFIG_ITEM_ID_DB_USER, "DB_User", "perfparse", _("Database user"), _("Database user"), "-U", "--db-user", TYPE_STRING, 1);
	config_item_add(config_definition, CONFIG_ITEM_ID_DB_PWD, "DB_Pass", "perfparse", _("Database password"), _("Database password"), "-P", "--db-password", TYPE_STRING, 1);
	config_item_add(config_definition, CONFIG_ITEM_ID_DB_DATABASE, "DB_Name", "perfparse", _("Database name"), _("Database name"), "-D", "--db-name", TYPE_STRING, 1);
	config_item_add(config_definition, CONFIG_ITEM_ID_DB_HOST, "DB_Host", "127.0.0.1", _("Database hostname"), _("Database hostname"), "-H", "--db-host", TYPE_STRING, 1);
	config_item_add(config_definition, 	CONFIG_ITEM_ID_STORAGE_SOCKET_OUTPUT_SOCKET_HOST, "Storage_Socket_Output_Host_Name", "localhost", _("Host name for Socket_output storage module"), _("Host name for Socket_output storage module"), NULL, "--socket_output_host", TYPE_STRING, 1);
	config_item_add(config_definition, 	CONFIG_ITEM_ID_STORAGE_SOCKET_OUTPUT_SOCKET_PORT, "Storage_Socket_Output_Port", "1974", _("Port for Socket_output storage module"), _("Port for Socket_output storage module"), NULL, "--socket_output_port", TYPE_INT, 1);
	config_item_add(config_definition, 	CONFIG_ITEM_ID_DUMMY_HOSTNAME, "Dummy_Hostname", my_gethostname(), _("Dummy hostname"), _("Dummy hostname if gethostname() does not work"), NULL, NULL, TYPE_STRING, 1);
	config_item_add(config_definition, 	CONFIG_ITEM_ID_NO_RAW_DATA, "No_Raw_Data", "no", _("Don't store raw data"), _("Don't store raw data"), NULL, "--no-raw-data",TYPE_BOOLEAN, 1);
	config_item_add(config_definition, 	CONFIG_ITEM_ID_NO_BIN_DATA, "No_Bin_Data", "no", _("Don't store bin data"), _("Don't store bin data"), NULL, "--no-bin-data",TYPE_BOOLEAN, 1);
	config_item_add(config_definition, 	CONFIG_ITEM_ID_DBTOOL_UPDATE, "DBTool_Update", "no", _("Update Database Schema"), _("Update Database Schema"), NULL, "--update",TYPE_BOOLEAN, 0);
	config_item_add(config_definition, 	CONFIG_ITEM_ID_DBTOOL_INSTALL, "DBTool_Install", "no", _("Install Dabatase Schema"), _("Store raw data"), NULL, "--install",TYPE_BOOLEAN, 0);
	config_item_add(config_definition, 	CONFIG_ITEM_ID_SHOW_CONFIG, "Show_Config", "no", _("Show config"), _("Show config"), NULL, "--show_config",TYPE_BOOLEAN, 0);
	/* xgettext: no-c-format (please don't remove this comment : xgettext needs it for translations) */
	config_item_add(config_definition, 	CONFIG_ITEM_ID_DBTOOL_RESET_HOST_DELETE_POLICY, "DBTool_Reset_Host_Policy", "%", _("Reset Host Delete Policy, use '%' for all hosts"), _("Reset Host Delete Policy, use '%' for all hosts"), NULL, "--reset-host-delete-policy", TYPE_STRING, 1);
	config_item_add(config_definition,  CONFIG_ITEM_ID_NO_TRANSACTIONS, "No_Transactions", "no", _("Dissable database transactions"), _("Dissable database transactions"), NULL, "--no_transactions", TYPE_BOOLEAN, 0);

	for(i=0; i<CONFIG_ITEM_ID_NB; i++) assert(config_definition[i].key != NULL);
}

void config_print_version(FILE*fd, char*header, char*footer) {
	fprintf(fd, _("%s\nVersion: %s (built %s %s)\n%s\n"), header, VERSION, __DATE__, __TIME__, footer);
}

void config_print_help(FILE*fd, char*header, char*footer, int*options) {
	int i;
	if(header) fprintf(fd, "%s", header);
	for(i=0; options[i]!=CONFIG_ITEM_ID_NB; i++) {
		if(config_definition[options[i]].long_option) {
			char str[64]; /* Should be OK for what we need : an option and a type */
			switch(config_definition[options[i]].type) {
				case TYPE_STRING : snprintf(str,64,_("%s <string>"), config_definition[options[i]].long_option); break;
				case TYPE_INT : snprintf(str,64,_("%s <integer>"), config_definition[options[i]].long_option); break;
				default : snprintf(str,64,"%s", config_definition[options[i]].long_option); 
			}
			if(config_definition[options[i]].short_option)
				fprintf(fd, "  %-2s, %-30s : %s\n",config_definition[options[i]].short_option,str, config_definition[options[i]].short_description);
			else
				fprintf(fd, "  %-34s : %s\n",str, config_definition[options[i]].short_description);
		}
	}
	if(footer) fprintf(fd, "%s\n", footer);
}

void config_show_config(FILE*fd, char*header, char*footer, int*options) {
	int i;
	if(header) fprintf(fd, "%s", header);
	for(i=0; options[i]!=CONFIG_ITEM_ID_NB; i++) {
		if(config_definition[options[i]].value && config_definition[options[i]].in_config_file) {
			if(config_definition[options[i]].long_description)
				fprintf(fd, "# %s\n",config_definition[options[i]].long_description);
			switch(config_definition[options[i]].type) {
				case TYPE_STRING : fprintf(fd, _("# %s = \"string\"\n"),config_definition[options[i]].key); break;
				case TYPE_INT : fprintf(fd, _("# %s = \"value\"\n"),config_definition[options[i]].key); break;
				case TYPE_BOOLEAN : fprintf(fd, "# %s = \"Y/N\"\n",config_definition[options[i]].key); break;
			}
			fprintf(fd, "%s = \"%s\"\n\n",config_definition[options[i]].key, config_definition[options[i]].value);
		}
	}
	if(footer) fprintf(fd, "%s\n", footer);
}

void config_load(char*config_file) {
	FILE *f;
	char sIn[1025];
	regex_t *pattern;
	int err;

	if ((f = fopen(config_file, "r")) == NULL) {
		fprintf(stderr, _("Error, can't open configuation file: \"%s\"\n"), config_file);
		exit(EXIT_FAILURE);
	}

	pattern = malloc(sizeof(regex_t));
	err = regcomp(pattern,"^[ \t]*([^= \t]+)[ \t]*=[ \t]*\"([^\"]*)\".*$",REG_EXTENDED);
	assert(0 == err);

	while (fgets(sIn, 1024, f)) {
		char*a;
		regmatch_t pmatch[3];

		if((a=strchr(sIn,'#'))) a[0] = '\0';
		if(!regexec(pattern,sIn,3,pmatch,0)) {
			int i;
			int ln = pmatch[1].rm_eo-pmatch[1].rm_so;
			int lv = pmatch[2].rm_eo-pmatch[2].rm_so;
			char *name = malloc(sizeof(char)*(1+ln));
			char *value = malloc(sizeof(char)*(1+lv));
			memcpy(name,sIn+pmatch[1].rm_so,ln);
			memcpy(value,sIn+pmatch[2].rm_so,lv);
			name[ln]='\0';
			value[lv]='\0';

			for(i=0; i<CONFIG_ITEM_ID_NB; i++) {
				if(!strcmp(name, config_definition[i].key)) {
					free(config_definition[i].value);
					config_definition[i].value = value;
					config_definition[i].integer = strtol(value,NULL,10);
					config_definition[i].boolean = strtoboolean(value);
					break;
				}
			}
			if(CONFIG_ITEM_ID_NB == i) free(value);
			
		}
	}
	fclose(f);
	regfree(pattern);
}

void config_argv(int argc, char**argv, char*header, char*footer, int*options) {
	int i,j;
	for(i=1; i<argc; i++) {
		if(!strcmp("-c",argv[i])) { i++; continue;}
		if(!strcmp("--config-file",argv[i])) { i++; continue; }
		if(!strcmp("--show_config",argv[i])) continue;
		if(!strcmp("--help",argv[i])) continue;
		if(!strcmp("-h",argv[i])) continue;
		if(!strcmp("--version",argv[i])) continue;
		if(!strcmp("-v",argv[i])) continue;

		for(j=0; j<CONFIG_ITEM_ID_NB; j++) {
			if( ( config_definition[j].short_option && (!strcmp(argv[i], config_definition[j].short_option))) || ( config_definition[j].long_option && (!strcmp(argv[i], config_definition[j].long_option)))) {
				if(TYPE_BOOLEAN == config_definition[j].type) {
					free(config_definition[j].value);
					config_definition[j].value = strdup("yes");
					config_definition[j].integer = 1;
					config_definition[j].boolean = 1;
				} else {
					i++;
					if(i < argc) {
						free(config_definition[j].value);
						config_definition[j].value = strdup(argv[i]);
						config_definition[j].integer = strtol(argv[i],NULL,10);
						config_definition[j].boolean = strtoboolean(argv[i]);
					} else {
						gchar*str = g_strdup_printf(_("Missing value for option %s\n\n%s"),argv[i-1], footer);
						config_print_help(stdout, header, str,options);
						g_free(str);
						exit(EXIT_FAILURE);
					}
				}
				break;
			}
		}
		if(CONFIG_ITEM_ID_NB == j) {
					gchar*str = g_strdup_printf(_("Unrecognized argument %s\n\n%s"), argv[i], footer);
					config_print_help(stdout, header, str,options);
					g_free(str);
			exit(EXIT_FAILURE);
		}
	}
}

config_option_e config_parse_config(int argc, char**argv, char*header, char*footer, int*options, const char*default_error_log) {
	int i;
	char*config_file = "/" SYSCONFDIR "/" CONFIG_FILE;
	int show_config = 0;

	config_definition_init(default_error_log);

	for(i=0; i<argc; i++) {
		if(!strcmp("-c",argv[i]) && (i< argc-1)) config_file = argv[i+1];
		if(!strcmp("--config-file",argv[i]) && (i< argc-1)) config_file = argv[i+1];
		if(!strcmp("--show_config",argv[i])) show_config = 1;
		if(!strcmp("--help",argv[i])) return(CONFIG_OPTION_HELP);
		if(!strcmp("-h",argv[i])) return(CONFIG_OPTION_HELP);
		if(!strcmp("--version",argv[i])) return(CONFIG_OPTION_VERSION); 
		if(!strcmp("-v",argv[i])) return(CONFIG_OPTION_VERSION);
	}
	config_load(config_file);
	config_argv(argc,argv,header,footer,options);
	if(show_config) return(CONFIG_OPTION_SHOW_CONFIG);
	return(CONFIG_OPTION_NONE);
}


void config_set_value_as_string(config_item_id_e id, const char*str) {
	if(config_definition[id].value) free(config_definition[id].value);
	config_definition[id].value = strdup(str);
	config_definition[id].boolean = strtoboolean(str);
}
