/*****************************************************************************
 *
 * storage_file_output.c - Parse Nagios data and store in relational database.
 *
 * Copyright (c) 2004 Yves Mettier (ymettier@libertysurf.fr)
 * Last Modified:   2004-09-16
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#define STORAGE_FILE_OUTPUT_VERSION "0.0.1"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <time.h>
#include <glib.h>
#include <libpp_common.h>
#include <libnagios_perfdata_parser.h>
#include "libpp_storage.h"
#include <config.h>
#include <gettext.h>

static guint log_date=0;
static FILE*log_fd=NULL;

int mkdir_p_for_file(char*filename) {
	char*pos;
	for(pos=filename; pos[0] == '/'; pos++);
	while(pos && pos[0]) {
		pos = strchr(pos,'/');
		if(pos) {
			pos[0] = '\0';
			if(mkdir (filename,0755) && (errno != EEXIST)) {
				pp_log(__FILE__,__LINE__, _("Could not create directory '%s'"), filename);
				pos[0] = '/';
				return(-1);	
			}
			pos[0] = '/';
			while(pos[0] == '/') pos++;
		}
	}
	return(0);	
}

int output_log_file_open(time_t tm) {
	gchar *path;
	char date[] = "YYYY/MM/YYYYMMDD.log";
	if(config_get_value_as_boolean(CONFIG_ITEM_ID_STORAGE_FILE_OUTPUT_LOG_ROTATE)) {
		struct tm *stm = localtime(&tm);
		strftime(date, sizeof(date), "%Y/%m/%Y%m%d.log",stm);
		path = g_strdup_printf("%s/%s",config_get_value_as_string(CONFIG_ITEM_ID_STORAGE_FILE_OUTPUT_LOG_FILENAME), date);
		mkdir_p_for_file(path);
		stm->tm_hour = 0;
		stm->tm_min = 0;
		stm->tm_sec = 0;
		log_date = mktime(stm);
	} else {
		path = g_strdup_printf("%s",config_get_value_as_string(CONFIG_ITEM_ID_STORAGE_FILE_OUTPUT_LOG_FILENAME));
	}

	log_fd = fopen(path,"a");
	g_free(path);

	return(log_fd?0:-1);
}

int storage_output_log_file_store_line(storage_module_t *this, const log_line_t *l) {
	if(config_get_value_as_boolean(CONFIG_ITEM_ID_STORAGE_FILE_OUTPUT_LOG_ROTATE)) {
		struct tm *stm = localtime(&(l->time));
		stm->tm_hour = 0;
		stm->tm_min = 0;
		stm->tm_sec = 0;
		if(mktime(stm) != log_date) {
			if(log_fd) fclose(log_fd);
			log_fd = NULL;
		}
	}
	if(NULL == log_fd) {
		if(output_log_file_open(l->time)) return(-1);
	}
	fprintf(log_fd,"%ld\t%s\t%s\t%s\t%s\t%s\n",l->time,
			l->macro[LOG_LINE_HOSTNAME],
			l->macro[LOG_LINE_SERVICEDESC],
			l->macro[LOG_LINE_OUTPUT],
			l->macro[LOG_LINE_SERVICESTATE],
			l->macro[LOG_LINE_PERFDATA]);
	fflush(log_fd);
	return(0);
}

int storage_output_log_file_connect(storage_module_t*this) {
	return(0);
}

int storage_output_log_file_disconnect(storage_module_t*this) {
	if(log_fd) fclose(log_fd);
	log_fd = NULL;
	return(0);
}

void storage_output_log_file_atexit(struct storage_module_t *sm) {
	if(log_fd) fclose(log_fd);
	log_fd = NULL;
}

void storage_file_output_print_usage(storage_module_t *this, FILE*fd, const char*id) {
	int options[] = {
		CONFIG_ITEM_ID_STORAGE_FILE_OUTPUT_LOG_FILENAME,
		CONFIG_ITEM_ID_STORAGE_FILE_OUTPUT_LOG_ROTATE,
		CONFIG_ITEM_ID_STORAGE_FILE_OUTPUT_LOG_KEEP_N_DAYS,
		CONFIG_ITEM_ID_NB
	};
	if(!strcmp(id,"version")) {
		fprintf(fd, _("Storage Module : %s\nVersion: %s (built %s %s)\n"),
				this->id, this->version, __DATE__, __TIME__);
	} else if(!strcmp(id, "help")) {
		fprintf(fd, _("Storage Module : %s\n"),this->id);
		config_print_help(fd, NULL,"", options);
	} else if(!strcmp(id, "show_config")) {
		fprintf(fd, _("# Storage Module : %s\n"), this->id);
		fprintf(fd, _("# ==============================\n\n"));
		config_show_config(fd, NULL,"", options);
		fprintf(fd, "\n");
	}
}


void storage_file_output_periodic_cleanup(storage_module_t *this) {

	check_dir_for_files_to_remove_or_compress(config_get_value_as_integer(CONFIG_ITEM_ID_STORAGE_FILE_OUTPUT_LOG_KEEP_N_DAYS), config_get_value_as_string(CONFIG_ITEM_ID_STORAGE_FILE_OUTPUT_LOG_FILENAME),NULL);
}

storage_module_t* storage_module_init(void) {
	storage_module_t *sm;
	sm = storage_new("File_Output",STORAGE_FILE_OUTPUT_VERSION);
	if(NULL == sm) return(NULL);
	sm->print_usage = storage_file_output_print_usage;
	sm->store_line = storage_output_log_file_store_line;
	sm->atexit = storage_output_log_file_atexit;
	sm->connect = storage_output_log_file_connect;
	sm->disconnect = storage_output_log_file_disconnect;
	sm->periodic_cleanup = storage_file_output_periodic_cleanup;
	sm->enabled = 1;

	log_date = 0;
	log_fd = NULL;
	return(sm);
}

