/*****************************************************************************
 *
 * storage_print.c - Parse Nagios data and store in relational database.
 *
 * Copyright (c) 2004 Yves Mettier (ymettier@libertysurf.fr)
 * Last Modified:   2004-09-01
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#define STORAGE_SOCKET_OUTPUT_VERSION "0.0.1"


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <signal.h>
#include <libnagios_perfdata_parser.h>
#include "libpp_storage.h"

#include <libpp_common.h>
#include <config.h>
#include <gettext.h>

int socket_id = -1;

int storage_socket_output_store_line(storage_module_t *this, const log_line_t *log_data) {
	if(socket_id > 0) {
		int i;
		for(i=0; i < LOG_LINE_MACRO_NB; i++) {
			write(socket_id, log_data->macro[i], strlen(log_data->macro[i]));
			if(i < LOG_LINE_MACRO_NB - 1) write(socket_id, "\t", 1);
		}
		write(socket_id, "\n", 1);
	} else {
		pp_log(__FILE__,__LINE__, _("Socket not connected"));
		return(-1);
	}
	return(0);
}

int storage_socket_output_store_perf_values(storage_module_t *this, const log_line_t *log_data, const perf_fields_t *perf_fields, const perf_value_t *perf_value) {
	return(0);
}

int storage_socket_output_set_registry(storage_module_t *this, const char*hostname, const char*key, const char*value, time_t tm) {
	if(socket_id > 0) {
		char*tmp;
		int l;
		l = sizeof("set_registry ") + 10 + strlen(hostname)+strlen(key)+strlen(value)+13;
		tmp = malloc(sizeof(char)*l);
		if(NULL == tmp) {
			pp_log(__FILE__,__LINE__, _("Not enough memory"));
			exit(EXIT_FAILURE);
		}
		snprintf(tmp,l,"set_registry %ld '%s' '%s' '%s'\n", tm,hostname,key,value);
		write(socket_id, tmp, strlen(tmp));
		free(tmp);
	} else {
		pp_log(__FILE__,__LINE__, _("Socket not connected"));
		return(-1);
	}
	return(0);
}

void socket_close(void) {
	if(-1 != socket_id) {
		shutdown(socket_id, 2);
		close(socket_id);
	}
	socket_id = -1;
}

int storage_socket_output_disconnect(struct storage_module_t *this) {
	socket_close();
	return(0);
}

void sigpipe_handler(int sig) {
	if(sig == SIGPIPE) {
		socket_close();
		pp_log(__FILE__,__LINE__, _("Connection break (SIGPIPE caught)"));
	}
}

void sigpipe_install_handler (void)
{
	struct sigaction action;
	action.sa_handler = sigpipe_handler;
	sigemptyset (&(action.sa_mask));
	action.sa_flags = 0;
	sigaction (SIGPIPE, &action, NULL);
}

int storage_socket_output_connect(struct storage_module_t *this) {
	char*host = config_get_value_as_string(CONFIG_ITEM_ID_STORAGE_SOCKET_OUTPUT_SOCKET_HOST);
	int port = config_get_value_as_integer(CONFIG_ITEM_ID_STORAGE_SOCKET_OUTPUT_SOCKET_PORT);
	/* Code below is from maitretarot (http://www.nongnu.org/maitretarot/)
	 * Please don't remove this note
	 * YM 20041019
	 */
	struct hostent server_address, *address;
	struct sockaddr_in *client_sockaddr_in;
	int optval = 1;

	if ((address = gethostbyname (host)) == NULL)
	{
		pp_log(__FILE__,__LINE__, _("Could not find host '%s'. Plugin storage_socket_output disabled"), host);
		this->enabled=0;
		socket_id = -1;
		return(-1);
	}

	memcpy (&server_address, address, sizeof (struct hostent));

	/* Open the server socket */

	if ((socket_id = socket (AF_INET, SOCK_STREAM, 6)) == -1)
	{
		pp_log(__FILE__,__LINE__, _("Could not create a new socket to host '%s'. Plugin storage_socket_output disabled"), host);
		this->enabled=0;
		socket_id = -1;
		return(-1);
	}

	setsockopt (socket_id, SOL_SOCKET, SO_REUSEADDR, &optval, sizeof (int));
	/* Get infos on the server */
	if ((client_sockaddr_in = malloc (sizeof (struct sockaddr_in))) == NULL)
	{
		pp_log(__FILE__,__LINE__, _("Could not allocate memory for new socket to host '%s'. Plugin storage_socket_output disabled"), host);
		this->enabled=0;
		socket_id = -1;
		return (-1);
	}

	client_sockaddr_in->sin_family = server_address.h_addrtype;
	client_sockaddr_in->sin_port = htons (port);
	memcpy ((char *) & (client_sockaddr_in->sin_addr.s_addr),
			server_address.h_addr, server_address.h_length);
	/* This works only on GNU/Linux?
	 *
	 bzero ((char *) & (client_sockaddr_in->sin_zero),
	 sizeof (struct sockaddr) - __SOCKADDR_COMMON_SIZE -
	 sizeof (unsigned short) - sizeof (struct in_addr));
	 *
	 */
	/* Connect to the server as a client */
	if (connect
			(socket_id, (struct sockaddr *) client_sockaddr_in,
			 sizeof (struct sockaddr_in)))
	{
		free (client_sockaddr_in);
		pp_log(__FILE__,__LINE__, _("Could not connect the socket to the server '%s'. Plugin storage_socket_output disabled"), host);
		this->enabled=0;
		socket_id = -1;
		return (-1);
	}
	free (client_sockaddr_in);
	sigpipe_install_handler ();

	return(socket_id);
}

void storage_socket_output_atexit(struct storage_module_t *this) {
	if(-1 != socket_id) {
		shutdown(socket_id, 2);
		close(socket_id);
	}
}

void storage_socket_output_print_statistics(storage_module_t *this, FILE*fd) {
}

void storage_socket_output_print_usage(storage_module_t *this, FILE*fd, const char*id) {
	int options[] = {
		CONFIG_ITEM_ID_STORAGE_SOCKET_OUTPUT_SOCKET_HOST,
		CONFIG_ITEM_ID_STORAGE_SOCKET_OUTPUT_SOCKET_PORT,
		CONFIG_ITEM_ID_NB
	};
	if(!strcmp(id,"version")) {
		fprintf(fd, _("Storage Module : %s\nVersion: %s (built %s %s)\n"),
				this->id, this->version, __DATE__, __TIME__);
	} else if(!strcmp(id, "help")) {
		fprintf(fd, _("Storage Module : %s\n"), this->id);
		config_print_help(fd, NULL,"", options);
	} else if(!strcmp(id, "show_config")) {
		fprintf(fd, _("# Storage Module : %s\n") ,this->id);
		fprintf(fd, _("# ==============================\n\n"));
		config_show_config(fd, NULL,"", options);
		fprintf(fd, "\n");
	}
}

storage_module_t* storage_module_init(void) {
	storage_module_t *sm;
	sm = storage_new("Socket_Output", STORAGE_SOCKET_OUTPUT_VERSION);
	if(NULL == sm) return(NULL);
	sm->connect = storage_socket_output_connect;
	sm->disconnect = storage_socket_output_disconnect;
	sm->store_line = storage_socket_output_store_line;
	sm->set_registry = storage_socket_output_set_registry;
	sm->store_perf_values = NULL;
	sm->atexit = storage_socket_output_atexit;
	sm->print_statistics = storage_socket_output_print_statistics;
	sm->print_usage = storage_socket_output_print_usage;
	sm->enabled = 1;
	return(sm);
}

