/*****************************************************************************
 *
 * perfparse-log2any.c - Parse Nagios data and store in relational database.
 *
 * Copyright (c) 2004 Yves Mettier (ymettier@libertysurf.fr)
 * Last Modified:   2004-09-01
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <signal.h>
#include <time.h>
#include <glib.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <config.h>
#include <sys/utsname.h>
#include "log_reader.h"
#include <libnagios_perfdata_parser.h>
#include "perfparse-common.h"

#include "storage.h"
#include <libpp_common.h>
#include <gettext.h>
#include <locale.h>

/* Statistic vars */

void print_global_statistics(FILE*fd, perfparse_statistics_t*stats) {

	float rate = 0;
	time_t tm = time(NULL);
	time_t delta = tm - stats->start_time;
	struct tm *tm_start = localtime(&(stats->start_time));	

	if (delta) rate = (float)stats->nb_lines_parsed / (float)delta;

	fprintf(fd, "+-----------------------------+   +-----------------------------+\n");
	fprintf(fd, "| Generic statistics          |   | Nb Log Lines :         %4d |\n", pp_log_count);
	fprintf(fd, "|   Nb lines parsed : %7d |   | Nb lines dropped :  %7d |\n", stats->nb_lines_parsed, stats->nb_lines_dropped);
	fprintf(fd, "|   Start date :   %04d-%02d-%02d |   | Elapsed time :     %02ld:%02ld:%02ld |\n", 
			tm_start->tm_year + 1900, tm_start->tm_mon + 1, tm_start->tm_mday,
			(delta/3600), ((delta%3600)/60), (delta%60));
	fprintf(fd, "|   Start time :     %02d:%02d:%02d |   | Rate : (line/sec)   %7.2f |\n", 
			tm_start->tm_hour, tm_start->tm_min, tm_start->tm_sec, rate);
	fprintf(fd, "+-----------------------------+   +-----------------------------+\n");
	if (pp_log_count > 1) {
		fprintf(fd, " *** WARNING! Log file has been written to.   Please check. ***\n");
		fprintf(fd, " *** Log file name: %s ***\n", pp_log_file);
	}
}

#ifdef RATE_DISPLAY
time_t rate_display(time_t last,perfparse_statistics_t*stats) {
	/* Rate Calculations */
	time_t now;
	int iSample = 2;
	static int iLastLine = 0;

	/* Rate Display */
	now = time(NULL);
	if (now - last > iSample) {
			stats->reading_rate = (double)(stats->nb_lines_parsed - iLastLine) / (double)(now - last);
		last = now;
		iLastLine = stats->nb_lines_parsed;
	}
	if (now != 0 && config_get_value_as_boolean(CONFIG_ITEM_ID_SHOW_STATUS_BAR)) {
#ifndef DEBUG
		printf("\r");
#endif /* DEBUG */
		printf(_("%02ld:%02ld:%02ld read %d @ %3.1f l/sec.  Met %d @ %.2f Met/l.  Drop %d. "), 
				(now - stats->start_time) / 3600, ((now - stats->start_time) / 60) % 60, (now - stats->start_time) % 60,
				stats->nb_lines_parsed, stats->reading_rate, 
				stats->nb_metrics_parsed, 
				(double)(stats->nb_metrics_parsed) / (double)stats->nb_lines_parsed,
				stats->nb_lines_dropped);
		fflush(stdout);
#ifdef DEBUG
		printf("\n");
#endif /* DEBUG */
	}
	return(last);

}
#else
#define rate_display(i,j) (0)
#endif

int main(int argc, char **argv)
{
	int options[] = {
		/* Don't put options that are specific to modules here */
		CONFIG_ITEM_ID_ERROR_LOG,
		CONFIG_ITEM_ID_ERROR_LOG_ROTATE,
		CONFIG_ITEM_ID_DROP_FILE,
		CONFIG_ITEM_ID_DROP_FILE_ROTATE,
		CONFIG_ITEM_ID_SERVICE_LOG,
		CONFIG_ITEM_ID_SERVICE_LOG_SAVE_POSITION,
		CONFIG_ITEM_ID_SERVICE_LOG_POSITION_MARK_PATH,
		CONFIG_ITEM_ID_HISTORY_START_TM,
		CONFIG_ITEM_ID_HISTORY_END_TM,
		CONFIG_ITEM_ID_SHOW_STATUS_BAR,
		CONFIG_ITEM_ID_DO_REPORT,
		CONFIG_ITEM_ID_DUMMY_HOSTNAME,
		CONFIG_ITEM_ID_NO_RAW_DATA,
		CONFIG_ITEM_ID_NO_BIN_DATA,
		CONFIG_ITEM_ID_STORAGE_MODULES_DIR,
		CONFIG_ITEM_ID_STORAGE_MODULES_LOAD,
		CONFIG_ITEM_ID_SHOW_CONFIG,
		CONFIG_ITEM_ID_NB
	};

	GString *log_line=g_string_sized_new(10);
	log_line_t *log_data=log_line_new();
	perf_fields_t *perf_fields = perf_fields_new();
	time_t rating_display_date_mark = time(NULL);
	perfparse_statistics_t stats = {0,0,0,0.,0};
	int log_fd_id;
	config_option_e config_option;
	char*service_log_source;
	setlocale(LC_ALL, "");
	bindtextdomain(PACKAGE, LOCALEDIR);
	textdomain(PACKAGE);
	stats.start_time = time(NULL);

	switch(config_option =config_parse_config(argc,argv,_("Perfparse-log2* [options]\n\n"),"\n",options, "stderr")) {
		case CONFIG_OPTION_HELP: config_print_help(stdout, _("Perfparse-log2* [options]\n\n"), "\n", options); break;
		case CONFIG_OPTION_VERSION: config_print_version(stdout, _("Perfparse-log2* [options]\n\n"), "\n"); break;
		case CONFIG_OPTION_SHOW_CONFIG: config_show_config(stdout, _("Perfparse-log2* [options]\n\n"), "\n", options); break;
		case CONFIG_OPTION_NONE: break;
	}

	if(config_get_value_as_string(CONFIG_ITEM_ID_SERVICE_LOG)[0] == '|') {
		pp_log(__FILE__,__LINE__,_("FIFO not authorized as the perf data input source"));
		exit(EXIT_FAILURE);
	}

	if(0 == storage_init(argv[0])) {
		pp_log(__FILE__,__LINE__,_("No module was loaded. Check the Storage_Module_Load config option."));
		exit(EXIT_FAILURE);
	}

	switch(config_option) {
		case CONFIG_OPTION_HELP: storage_print_usage(stdout, "help"); exit(EXIT_SUCCESS);
		case CONFIG_OPTION_VERSION: storage_print_usage(stdout, "version"); exit(EXIT_SUCCESS);
		case CONFIG_OPTION_SHOW_CONFIG: storage_print_usage(stdout, "show_config"); exit(EXIT_SUCCESS);
		case CONFIG_OPTION_NONE: break;
	}

	atexit(storage_atexit);

	storage_connect();

	registry_update();

	/* Open the log file */
	log_reader_install_signal_handler();
	service_log_source = config_get_value_as_string(CONFIG_ITEM_ID_SERVICE_LOG);
	if('\0' == service_log_source[0]) exit(EXIT_FAILURE);
	if((0 > (log_fd_id = open_log_source(service_log_source)))) {
		exit(EXIT_FAILURE);
	}

	log_source_command(log_fd_id);
	/* Read the log file. */
	while(1) {
		GString *l;
		log_reader_mask_signals();
		if(NULL == (l = log_reader(log_fd_id, log_line))) {
			log_reader_unmask_signals();
			break;
		}
		parse_and_store_perf_data_line(l,&stats, log_data, perf_fields);
		log_reader_unmask_signals();
		rating_display_date_mark = rate_display(rating_display_date_mark, &stats);
	}
	if(config_get_value_as_boolean(CONFIG_ITEM_ID_SHOW_STATUS_BAR))		printf("\n");
	close_log_source(log_fd_id, 1);
	log_reader_remove_signal_handler();
	storage_disconnect();
	storage_print_statistics(stdout,print_global_statistics,&stats);
	exit(EXIT_SUCCESS);
}

