/*****************************************************************************
 *
 * perfparsed.c - Parse Nagios data and store in relational database.
 *
 * Copyright (c) 2004 Yves Mettier (ymettier@libertysurf.fr)
 * Last Modified:   2004-09-01
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>
#include <time.h>
#include <glib.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/utsname.h>
#include "log_reader.h"
#include <libnagios_perfdata_parser.h>
#include "server_parser.h"
#include "perfparse-common.h"
#include "perfparsed.h"

#include "storage.h"
#include <libpp_common.h>
#include <config.h>
#include <gettext.h>
#include <locale.h>

void daemonize (const char*lock_file)
{
	int pid;
	int i;
	int fd;
	char str[12];

	pid = fork ();
	if (pid < 0)
	{
		pp_log(__FILE__,__LINE__, _("daemonize failed (fork problem : errno=%d msg=%s)"),errno,strerror(errno));
		exit(EXIT_FAILURE);
	}
	if (pid > 0)
	{
		exit (EXIT_SUCCESS);
	}

	setsid ();

	pid = fork ();
	if (pid < 0)
	{
		pp_log(__FILE__,__LINE__, _("daemonize failed (fork problem : errno=%d msg=%s)"),errno,strerror(errno));
		exit(EXIT_FAILURE);
	}
	if (pid > 0)
	{
		exit (EXIT_SUCCESS);
	}

	for (i = getdtablesize (); i >= 0; --i)
		close (i);

	i = open ("/dev/null", O_RDWR);
	dup (i);
	dup (i);

	umask (022);
	if(0 != chdir (LOCALSTATEDIR)) {
		pp_log(__FILE__,__LINE__, _("daemonize failed (could not chdir('%s') : errno=%d msg=%s)"),LOCALSTATEDIR, errno,strerror(errno));
		exit(EXIT_FAILURE);
	}

	fd = open (lock_file, O_RDWR | O_CREAT, 0640);
	if (fd < 0)
		exit (EXIT_FAILURE);
	if (lockf (fd, F_TLOCK, 0) < 0) {
		char tmp[20];
		int i;
		read(fd,tmp,20);
		for(i=0; i<20; i++) {
			if(tmp[i] == '\n') tmp[i] = '\0';
			if(tmp[i] == '\0') break;
		}
		tmp[i] = '\0';
		pid = strtol(tmp,NULL,10);
		if((0 == pid) && (EINVAL == errno)) {
			pp_log(__FILE__,__LINE__, _("File '%s' is corrupted"),lock_file);
			exit (EXIT_FAILURE);
		}
		pp_log(__FILE__,__LINE__, _("Perfparsed already running with PID %d"),pid);
		exit (EXIT_SUCCESS);
	}
	snprintf (str, 12, "%ld\n", (long)getpid ());
	write (fd, str, strlen (str));
}

int main(int argc, char **argv)
{
	int options[] = {
		/* Don't put options that are specific to modules here */
		CONFIG_ITEM_ID_ERROR_LOG,
		CONFIG_ITEM_ID_ERROR_LOG_ROTATE,
		CONFIG_ITEM_ID_ERROR_LOG_KEEP_N_DAYS,
		CONFIG_ITEM_ID_DROP_FILE,
		CONFIG_ITEM_ID_DROP_FILE_ROTATE,
		CONFIG_ITEM_ID_DROP_FILE_KEEP_N_DAYS,
		CONFIG_ITEM_ID_SERVER_PORT,
		CONFIG_ITEM_ID_SERVICE_LOG,
		CONFIG_ITEM_ID_SERVICE_LOG_SAVE_POSITION,
		CONFIG_ITEM_ID_SERVICE_LOG_POSITION_MARK_PATH,
		CONFIG_ITEM_ID_DAEMON_LOCK,
		CONFIG_ITEM_ID_DAEMONIZE,
		CONFIG_ITEM_ID_PERIODIC_CLEANUP,
		CONFIG_ITEM_ID_PERIODIC_CLEANUP_LOCK,
		CONFIG_ITEM_ID_PERIODIC_CLEANUP_HOUR,
		CONFIG_ITEM_ID_DUMMY_HOSTNAME,
		CONFIG_ITEM_ID_NO_RAW_DATA,
		CONFIG_ITEM_ID_NO_BIN_DATA,
		CONFIG_ITEM_ID_STORAGE_MODULES_DIR,
		CONFIG_ITEM_ID_STORAGE_MODULES_LOAD,
		CONFIG_ITEM_ID_STORAGE_MODULES_STATUS_FILE,
		CONFIG_ITEM_ID_SHOW_CONFIG,
		CONFIG_ITEM_ID_NB
	};
	GString *log_line=g_string_sized_new(10);
	log_line_t *log_data=log_line_new();
	perf_fields_t *perf_fields = perf_fields_new();
	char*service_log_source;
	config_option_e config_option;

	setlocale(LC_ALL, "");
	bindtextdomain(PACKAGE, LOCALEDIR);
	textdomain(PACKAGE);
	
	switch(config_option = config_parse_config(argc,argv,_("Perfparsed [options]\n\n"),"\n",options,LOCALSTATEDIR "/log/perfparsed.log")) {
		case CONFIG_OPTION_HELP: config_print_help(stdout, _("Perfparsed [options]\n\n"), "\n", options); break;
		case CONFIG_OPTION_VERSION: config_print_version(stdout, _("Perfparsed [options]\n\n"), "\n"); break;
		case CONFIG_OPTION_SHOW_CONFIG: config_show_config(stdout, _("Perfparsed [options]\n\n"), "\n", options); break;
		case CONFIG_OPTION_NONE: break;
	}

	if(!strcmp(config_get_value_as_string(CONFIG_ITEM_ID_SERVICE_LOG),"-")) {
		pp_log(__FILE__,__LINE__,_("Stdin not authorized as the perf data input source"));
		exit(EXIT_FAILURE);
	}
	if(config_get_value_as_string(CONFIG_ITEM_ID_SERVICE_LOG)[0] == '>') {
		pp_log(__FILE__,__LINE__,_("Client socket not authorized as the perf data input source"));
		exit(EXIT_FAILURE);
	}


	if(config_get_value_as_boolean(CONFIG_ITEM_ID_DAEMONIZE) && (config_option == CONFIG_OPTION_NONE)) {
		daemonize(config_get_value_as_string(CONFIG_ITEM_ID_DAEMON_LOCK));
		pp_log(__FILE__,__LINE__, _("Perfparsed successfully daemonized (pid=%ld)"),getpid());
	}

	if(0 == storage_init(NULL)) {
		pp_log(__FILE__,__LINE__,_("No module was loaded. Check the Storage_Module_Load config option."));
		exit(EXIT_FAILURE);
	}

	switch(config_option) {
		case CONFIG_OPTION_HELP: storage_print_usage(stdout, "help"); exit(EXIT_SUCCESS);
		case CONFIG_OPTION_VERSION: storage_print_usage(stdout, "version"); exit(EXIT_SUCCESS);
		case CONFIG_OPTION_SHOW_CONFIG: storage_print_usage(stdout, "show_config"); exit(EXIT_SUCCESS);
		case CONFIG_OPTION_NONE: break;
	}

	atexit(storage_atexit);
	storage_connect();
	storage_dump_status();

	registry_update();

	/* Open the log file */
	log_reader_install_signal_handler();
	service_log_source = config_get_value_as_string(CONFIG_ITEM_ID_SERVICE_LOG);
	if(service_log_source[0] && (0 > (open_log_source(service_log_source)))) {
		exit(EXIT_FAILURE);
	}
	if((config_get_value_as_integer(CONFIG_ITEM_ID_SERVER_PORT) > 0) && (0 > log_source_new_server(config_get_value_as_integer(CONFIG_ITEM_ID_SERVER_PORT)))) {
		exit(EXIT_FAILURE);
	}

	while(1) {
		int i;
		log_source_test_input();

		for(i=0; log_source_type(i) != LOG_FD_NOEXIST; i++) {
			GString *l;
			if(log_source_input_is_ready(i)) {
				switch(log_source_type(i)) {
					case LOG_FD_FILE:
					case LOG_FD_STDIN:
					case LOG_FD_PIPE:
					case LOG_FD_SOCKET:

						log_reader_mask_signals();
						if(NULL != (l = log_reader(i, log_line))) {
							/* Note : log_reader is supposed to return log_line or NULL but nothing else.
							 * Otherwise, this code contains a memory leak with l not freed
							 */
							if(0 < strtol(l->str,NULL,10)) 
								parse_and_store_perf_data_line(l,NULL,log_data,perf_fields);
							else if(log_source_type(i) == LOG_FD_SOCKET)
								parse_command(l,i);
							else
								log_dropped_line(l->str);
						}
						log_reader_unmask_signals();
						break;

					case LOG_FD_SOCKET_MAIN:
						log_source_new_connected_client(i);
						log_source_output_write(i,"Hello","\n");
						break;
					case LOG_FD_UNDEF :
						break;
					default :
						pp_log(__FILE__,__LINE__,_("This is a bug. This code should never be executed"));
				}
			}
		}
		storage_dump_status();
		storage_periodic_cleanup();
		while(0 < waitpid(-1,NULL,WNOHANG));
	}
	pp_log(__FILE__,__LINE__,_("This is a bug. This code should never be executed"));
	close_all_log_sources();
	log_reader_remove_signal_handler();
	storage_disconnect();
	exit(EXIT_SUCCESS);
}

