/*****************************************************************************
 *
 * storage.c - Parse Nagios data and store in relational database.
 *
 * Copyright (c) 2004 Yves Mettier (ymettier@libertysurf.fr)
 * Last Modified:   2004-09-01
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <dlfcn.h>
#include <sys/types.h>
#include <utime.h>
#include <libnagios_perfdata_parser.h>
#include <libpp_common.h>
#include <libpp_storage.h>
#include "perfparsed.h"
#include "storage.h"
#include <glib.h>

#include <config.h>
#include <gettext.h>

storage_module_element_t *storage_modules = NULL;
perf_value_t *perf_value;
int main_pid;
#ifdef PERFPARSED
time_t last_periodic_cleanup_tm;
unsigned int dump_status=0;
#endif


void storage_load(const char*storage_name) {
	storage_module_t *sm_new;
	storage_module_element_t *sm, *sme;
	char*libpath;
	storage_module_t *(*storage_module_init_cb)(void);
	void*storage_module_handle;

	libpath = g_strdup_printf("%s/libpp_storage_%s." SHLIBEXT,config_get_value_as_string(CONFIG_ITEM_ID_STORAGE_MODULES_DIR),storage_name);
	if(NULL == (storage_module_handle = dlopen(libpath, RTLD_LAZY))) { 
		pp_log(__FILE__,__LINE__, _("storage_%s module failed to load (msg='%s')\n"),storage_name, dlerror()); 
		exit(EXIT_FAILURE); 
	}
	g_free(libpath);

	storage_module_init_cb = dlsym(storage_module_handle, "storage_module_init");
	if(NULL == storage_module_init_cb) {
		pp_log(__FILE__,__LINE__, _("storage_%s module failed to initialize\n"),storage_name); 
		exit(EXIT_FAILURE); 
	}

	sm_new = storage_module_init_cb();
	if(NULL == sm_new) {
		pp_log(__FILE__,__LINE__, _("storage_%s module failed to initialize itself\n"),storage_name); 
		exit(EXIT_FAILURE); 
	}
	sm_new->log = pp_log;

	if(NULL == (sme=malloc(sizeof(storage_module_element_t)))) {
		pp_log(__FILE__,__LINE__, _("Not enough memory (malloc failed)\n")); 
		exit(EXIT_FAILURE); 
	}
	sme->module = sm_new;
	sme->handle = storage_module_handle;
	sme->next = NULL;

	if(storage_modules) {
		for(sm = storage_modules; sm->next; sm = sm->next);
		sm->next = sme;
		sme->prev = sm;
	} else {
		storage_modules = sme;
		storage_modules->prev = NULL;
	}
		pp_log(__FILE__,__LINE__, _("storage_%s module successfully loaded\n"),storage_name); 
}

int storage_init(const char*argv0) {
	const char*b;
	char*modules_list;
	char*ms, *me;
	char e;
	int n;
	storage_module_element_t*sm;
	
#ifdef PERFPARSED
	last_periodic_cleanup_tm = 0;
#endif

	main_pid = getpid();
	storage_modules = NULL;
	modules_list = strdup(config_get_value_as_string(CONFIG_ITEM_ID_STORAGE_MODULES_LOAD));
	perf_value = perf_value_new();

	if(argv0) {
		for(b=argv0;b[0]; b++);
		for(b--; (b>=argv0) && b[0] != '/'; b--);
		if(b[0] == '/') b++;

		if(strcmp(b,"perfparse-log2any") && !strncmp(b,"perfparse-log2",sizeof("perfparse-log2")-1)) {
			storage_load(b+sizeof("perfparse-log2")-1);
			return(storage_modules?1:0);
		}
	}
	if(modules_list[0] == '\0') return(0);
	ms = modules_list;
	e = ',';
	while(',' == e) {
		for(me=ms; (',' != me[0]) && ('\0' != me[0]); me++);
		e = me[0];
		me[0] = '\0';
		storage_load(ms);
		ms = me+1;
	}
	
	n=0;
	for(sm=storage_modules; sm; sm = sm->next) n++;
	return(n);
}

int storage_connect(void) {
	storage_module_element_t *this;
	for(this=storage_modules; this; this = this->next) {
		if(this->module->enabled && this->module->connect) this->module->connect(this->module);
	}

	return(0);
}

int storage_disconnect(void) {
	storage_module_element_t *this;
	for(this=storage_modules; this; this = this->next) {
		if(this->module->enabled && this->module->disconnect) this->module->disconnect(this->module);
	}

	return(0);
}

int storage_set_registry(const char*hostname, const char*key, const char*value, time_t tm) {
	storage_module_element_t *this;
	for(this=storage_modules; this; this = this->next) {
		if(this->module->enabled && this->module->set_registry) this->module->set_registry(this->module,hostname,key,value,tm);
	}
	return(0);
}

int storage_store_line(const log_line_t *log_line) {
	storage_module_element_t *this;
	for(this=storage_modules; this; this = this->next) {
		if(this->module->enabled) {
			if(this->module->store_line) {
				int r = this->module->store_line(this->module,log_line);
				if(0 != r) {
					pp_log(__FILE__,__LINE__, _("Could not store a line with storage module '%s' (check permissions, connection parameters...)"), this->module->id);
#ifdef PERFPARSED
		exit(EXIT_FAILURE);
#endif
				}
			}
		}
	}
	return(0);
}

int storage_store_perf(const log_line_t *log_line, const perf_fields_t *perf_fields) {
	storage_module_element_t *this;
	int perf = 0;
	if(perf_fields->type == PERF_FIELDS_TYPE_PERF) {
		if(!perfstring_to_perfvalue(perf_fields->value,perf_value)) perf = 1; 
	}
	for(this=storage_modules; this; this = this->next) {
		if(this->module->enabled) {
			if(perf && this->module->store_perf_values) this->module->store_perf_values(this->module,log_line,perf_fields,perf_value);
		}
	}
	return(0);
}

void storage_atexit(void) {
	storage_module_element_t *this;
	if(main_pid == getpid()) {
	for(this=storage_modules; this; this = this->next) {
		if(this->module->enabled) {
			if(this->module->atexit) this->module->atexit(this->module);
		}
	}
#ifdef PERFPARSED
	pp_log(__FILE__,__LINE__, _("Perfparsed stopped\n"));
#endif
	}
}

int storage_print_statistics(FILE*fd, void(*print_global_statistics_cb)(FILE*fd, perfparse_statistics_t*stats), perfparse_statistics_t*stats) {
	storage_module_element_t *this;
	if (!config_get_value_as_boolean(CONFIG_ITEM_ID_DO_REPORT)) return(0);
	if(print_global_statistics_cb) print_global_statistics_cb(fd,stats);
	fprintf(fd,"\n");
	for(this=storage_modules; this; this = this->next) {
		if(this->module->enabled && this->module->print_statistics) this->module->print_statistics(this->module,fd);
	}
	return(0);
}

int storage_print_usage(FILE*fd, const char*id) {
	storage_module_element_t *this;
	for(this=storage_modules; this; this = this->next) {
		if(this->module->enabled && this->module->print_usage) this->module->print_usage(this->module,fd,id);
	}
	return(0);
}

#ifdef PERFPARSED
int storage_periodic_cleanup(void) {
	storage_module_element_t *this;
	long h;
	time_t t;
	struct tm *tm;

	if (!config_get_value_as_boolean(CONFIG_ITEM_ID_PERIODIC_CLEANUP)) return(0);
	/* Check if we ran this today */
	h = config_get_value_as_integer(CONFIG_ITEM_ID_PERIODIC_CLEANUP_HOUR);
	t = time(NULL);
	tm = localtime(&t);
	tm->tm_sec = 0;
	tm->tm_min = h%100;
	tm->tm_hour = h/100;
	t = mktime(tm);
	if((time(NULL) > t) && (t > last_periodic_cleanup_tm)) {
		int pid;
		last_periodic_cleanup_tm = t;

		if(config_get_value_as_boolean(CONFIG_ITEM_ID_DAEMONIZE)) {
			switch(pid = fork()) {
				case -1:
					pp_log(__FILE__,__LINE__, _("Perfparsed : periodic cleanup cannot start (fork failed)\n"));
					break;
				case 0:
					daemonize(config_get_value_as_string(CONFIG_ITEM_ID_PERIODIC_CLEANUP_LOCK));
					break;
				default :
					return(0);
			}
		}
		pp_log(__FILE__,__LINE__, _("Perfparsed : periodic cleanup\n"));
		if(config_get_value_as_boolean(CONFIG_ITEM_ID_ERROR_LOG_ROTATE)) {
			/* Error log */
			char*dir, *pattern;
			dir = g_path_get_dirname(config_get_value_as_string(CONFIG_ITEM_ID_ERROR_LOG));
			pattern = g_path_get_basename(config_get_value_as_string(CONFIG_ITEM_ID_ERROR_LOG));
			check_dir_for_files_to_remove_or_compress(config_get_value_as_integer(CONFIG_ITEM_ID_ERROR_LOG_KEEP_N_DAYS),dir,pattern);
			g_free(dir);
			g_free(pattern);
		}
		if(config_get_value_as_boolean(CONFIG_ITEM_ID_DROP_FILE_ROTATE)) {
			/* Drop file */
			check_dir_for_files_to_remove_or_compress(config_get_value_as_integer(CONFIG_ITEM_ID_DROP_FILE_KEEP_N_DAYS),config_get_value_as_string(CONFIG_ITEM_ID_DROP_FILE),NULL);
		}
		/* Module periodic_cleanup procedures */
		for(this=storage_modules; this; this = this->next) {
			if(this->module->enabled && this->module->periodic_cleanup) this->module->periodic_cleanup(this->module);
		}
		if(config_get_value_as_boolean(CONFIG_ITEM_ID_DAEMONIZE)) {
			exit(EXIT_SUCCESS);
		}
	}
	return(0);
}
#endif

#ifdef PERFPARSED
void storage_dump_status(void) {
	char*file;
	FILE*fd;
	unsigned int status = 0;
	storage_module_element_t *this;
	if(NULL == (file = config_get_value_as_string(CONFIG_ITEM_ID_STORAGE_MODULES_STATUS_FILE))) return;
	if(file[0] == '\0') return;

	for(this=storage_modules; this; this = this->next) {
		status = status << 1;
		if(this->module->enabled) status |= 1;
	}
	if(dump_status != status) {
		dump_status = status;

		if(NULL == (fd = fopen(file,"w"))) {
			pp_log(__FILE__,__LINE__,_("Could not dump modules status\n"));
			return;
		} else {
			for(this=storage_modules; this; this = this->next) {
				if(this->module->enabled) fprintf(fd, "%s:enabled\n",this->module->id);
				else fprintf(fd,"%s:disabled\n", this->module->id);
			}
			fclose(fd);
		}
	}

	/* Watchdog feature : touch the file. If not touched for a while : maybe a problem ? */
	utime(file,NULL);

}
#endif
